"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _nullishCoalesce(lhs, rhsFn) { if (lhs != null) { return lhs; } else { return rhsFn(); } } function _optionalChain(ops) { let lastAccessLHS = undefined; let value = ops[0]; let i = 1; while (i < ops.length) { const op = ops[i]; const fn = ops[i + 1]; i += 2; if ((op === 'optionalAccess' || op === 'optionalCall') && value == null) { return undefined; } if (op === 'access' || op === 'optionalAccess') { lastAccessLHS = value; value = fn(value); } else if (op === 'call' || op === 'optionalCall') { value = fn((...args) => value.call(lastAccessLHS, ...args)); lastAccessLHS = undefined; } } return value; }var _cleaners = require('cleaners');









var _types = require('edge-core-js/types');
var _ethers = require('ethers');






var _swapHelpers = require('../../../../util/swapHelpers');
var _utils = require('../../../../util/utils');

var _defiUtils = require('../../defiUtils');





var _uniV2Contracts = require('../uniV2Contracts');
var _uniV2Utils = require('../uniV2Utils');

const swapInfo = {
  pluginId: 'tombSwap',
  isDex: true,
  displayName: 'TombSwap',
  supportEmail: 'support@edge.app'
}

const asInitOptions = _cleaners.asObject.call(void 0, {
  quiknodeApiKey: _cleaners.asOptional.call(void 0, _cleaners.asString)
})

const EXPIRATION_MS = 1000 * 60
const SLIPPAGE = '0.05'

 function makeTombSwapPlugin(
  opts
) {
  const { quiknodeApiKey } = asInitOptions(opts.initOptions)
  const provider = _uniV2Contracts.getFtmProvider.call(void 0, quiknodeApiKey)

  const fetchSwapQuoteInner = async (
    request,
    uid
  ) => {
    const { fromWallet, toWallet, fromTokenId, toTokenId } = request

    // Sanity check: Both wallets should be of the same chain.
    if (
      fromWallet.currencyInfo.pluginId !== 'fantom' ||
      toWallet.currencyInfo.pluginId !== 'fantom'
    )
      throw new (0, _types.SwapCurrencyError)(swapInfo, request)

    // Parse input/output token addresses. If either from or to swap sources
    // are for the native currency, convert the address to the wrapped equivalent.
    const inOutAddresses = _defiUtils.getInOutTokenAddresses.call(void 0, 
      fromWallet.currencyConfig,
      _uniV2Contracts.WFTM_TOKEN_ADDRESS,
      fromTokenId,
      toTokenId
    )
    const { fromTokenAddress, toTokenAddress, isWrappingSwap } = inOutAddresses

    // Calculate swap amounts
    const tombSwapRouter = _uniV2Contracts.makeTombSwapRouterContract.call(void 0, provider)
    const { amountToSwap, expectedAmountOut } = await _uniV2Utils.getSwapAmounts.call(void 0, 
      tombSwapRouter,
      request,
      swapInfo,
      [fromTokenAddress, toTokenAddress],
      isWrappingSwap
    )

    // Generate swap transactions
    const toAddress = (await toWallet.getReceiveAddress({ tokenId: null }))
      .publicAddress
    const expirationDate = new Date(Date.now() + EXPIRATION_MS)
    const deadline = Math.round(expirationDate.getTime() / 1000) // unix timestamp
    const customNetworkFee = _swapHelpers.customFeeCache.getFees(uid)
    const path = [fromTokenAddress, toTokenAddress]
    const wrappedFtmContract = _uniV2Contracts.makeWrappedFtmContract.call(void 0, provider)
    const swapTxs = await _uniV2Utils.getSwapTransactions.call(void 0, 
      provider,
      inOutAddresses,
      path,
      tombSwapRouter,
      wrappedFtmContract,
      amountToSwap,
      expectedAmountOut,
      toAddress,
      SLIPPAGE,
      deadline,
      _optionalChain([customNetworkFee, 'optionalAccess', _ => _.gasPrice])
    )

    const fromAddress = (await fromWallet.getReceiveAddress({ tokenId: null }))
      .publicAddress
    // toEdgeUnsignedTxs
    const edgeSpendInfos = swapTxs.map((swapTx, i) => {
      // Convert to our spendInfo
      const edgeSpendInfo = {
        tokenId: request.fromTokenId,
        spendTargets: [
          {
            memo: swapTx.data,
            nativeAmount:
              swapTxs.length === 2 && i === 0
                ? '0' // approval transactions don't have a value
                : amountToSwap,
            publicAddress: swapTx.to
          }
        ],
        customNetworkFee: {
          gasPrice:
            swapTx.gasPrice != null
              ? _ethers.ethers.utils.formatUnits(swapTx.gasPrice, 'gwei').toString()
              : '0',
          gasLimit: _nullishCoalesce(_optionalChain([swapTx, 'access', _2 => _2.gasLimit, 'optionalAccess', _3 => _3.toString, 'call', _4 => _4()]), () => ( '0'))
        },
        networkFeeOption: 'custom'
      }

      return edgeSpendInfo
    })

    let spendInfo = edgeSpendInfos[0]
    let preTx
    if (edgeSpendInfos.length > 1) {
      spendInfo = edgeSpendInfos[1]
      const approvalSpendInfo = {
        ...edgeSpendInfos[0],
        assetAction: {
          assetActionType: 'tokenApproval'
        },
        savedAction: {
          actionType: 'tokenApproval',
          tokenApproved: {
            pluginId: fromWallet.currencyInfo.pluginId,
            tokenId: fromTokenId,
            nativeAmount: amountToSwap
          },
          tokenContractAddress: inOutAddresses.fromTokenAddress,
          contractAddress: tombSwapRouter.address
        }
      }

      preTx = await request.fromWallet.makeSpend(approvalSpendInfo)
    }

    spendInfo = {
      ...spendInfo,
      assetAction: {
        assetActionType: 'swap'
      },
      savedAction: {
        actionType: 'swap',
        swapInfo,
        isEstimate: false,
        toAsset: {
          pluginId: request.toWallet.currencyInfo.pluginId,
          tokenId: request.toTokenId,
          nativeAmount: expectedAmountOut.toString()
        },
        fromAsset: {
          pluginId: request.fromWallet.currencyInfo.pluginId,
          tokenId: request.fromTokenId,
          nativeAmount: amountToSwap
        },
        payoutAddress: toAddress,
        payoutWalletId: request.toWallet.id,
        refundAddress: fromAddress
      }
    }

    _swapHelpers.customFeeCache.setFees(uid, spendInfo.customNetworkFee)

    return {
      request,
      spendInfo,
      swapInfo,
      fromNativeAmount: amountToSwap,
      expirationDate,
      preTx
    }
  }

  const out = {
    swapInfo,
    async fetchSwapQuote(req) {
      const request = _utils.convertRequest.call(void 0, req)

      const uid = _swapHelpers.customFeeCache.createUid()

      const newRequest = await _swapHelpers.getMaxSwappable.call(void 0, 
        fetchSwapQuoteInner,
        request,
        uid
      )
      const swapOrder = await fetchSwapQuoteInner(newRequest, uid)
      return await _swapHelpers.makeSwapPluginQuote.call(void 0, swapOrder)
    }
  }

  return out
} exports.makeTombSwapPlugin = makeTombSwapPlugin;
